from django.db import models
from django.utils.safestring import mark_safe

from orderable.models import Orderable

class Option(Orderable):
    """
    User options (op-ins)
    """
    name = models.CharField(max_length=150)
   
    class Meta:
        app_label = 'profiles'
        ordering = ('sort_order',)
   
    def __unicode__(self):
        return mark_safe(self.name)




from django.db import models
from django.utils.translation import ugettext_lazy as _

def register(cls, admin_cls):
    cls.add_to_class('options', models.ManyToManyField('profiles.Option', verbose_name=_('Opt-ins'), null=True, blank=True))

    if admin_cls:
        admin_cls.list_display_filter += ['options', ]

        if admin_cls.fieldsets:
            admin_cls.fieldsets.append((_('Opt-ins'), {
                    'fields': ['options',],
                    'classes': ('collapse',),
                }))
            admin_cls.filter_horizontal = admin_cls.filter_horizontal + ('options',)



from django.contrib import admin

from orderable.admin import OrderableAdmin

from .models import Notification


class NotificationOptions(OrderableAdmin):
    prepopulated_fields = {'slug': ('name',)}
    fields = ('name', 'slug', 'field_name', )
    list_display = ('name', 'slug', 'field_name', 'sort_order_display')
    search_fields = ('name', )
    ordering = ('sort_order',) 

admin.site.register(Notification, NotificationOptions)





from django.db import models
from django.utils.translation import ugettext_lazy as _


def register(cls, admin_cls):
    cls.add_to_class('modified', models.DateTimeField(auto_now=True))

    if admin_cls:

        admin_cls.readonly_fields += ['modified']
        admin_cls.list_display += ['modified']
        if admin_cls.fieldsets:
            other_title = _('Other options')
            try:
                other_set = (s for s in admin_cls.fieldsets if lambda s: s[0] == other_title).next()
            except StopIteration:
                fields = admin_cls.fieldsets[0][1]['fields']
            else:
                fields = other_set[1]['fields']
            fields.append('modified')



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.utils.translation import ugettext_lazy as _


if 'countries' not in settings.INSTALLED_APPS:
    raise ImproperlyConfigured("The address extension requires incuna-countries (http://github.com/incuna/incuna-countries) - ensure that 'countries' is added to your INSTALLED_APPS.")


def register(cls, admin_cls):
    cls.add_to_class('address1', models.CharField(max_length=255, verbose_name=_('address'), null=True, blank=True))
    cls.add_to_class('address2', models.CharField(max_length=255, verbose_name=_('address 2'), null=True, blank=True))
    cls.add_to_class('city', models.CharField(max_length=255, verbose_name=_('town/city'), null=True, blank=True))
    cls.add_to_class('region', models.CharField(max_length=255, verbose_name=_('county/state/province'), null=True, blank=True))
    cls.add_to_class('postcode', models.CharField(max_length=15, verbose_name=_('postcode'), null=True, blank=True))
    cls.add_to_class('country', models.ForeignKey('countries.Country', null=True, blank=True))
    cls.add_to_class('telephone', models.CharField(max_length=32, verbose_name=_('telephone'), null=True, blank=True))

    if admin_cls:
        admin_cls.search_fields += ['address1', 'address2', 'city', 'region', 'postcode', 'telephone']
        admin_cls.list_display_filter += ['country', ]

        if admin_cls.fieldsets:
            admin_cls.fieldsets.append((_('Address'), {
                    'fields': ['address1', 'address2','city', 'region', 'postcode', 'country', 'telephone'],
                    'classes': ('collapse',),
                }))




from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse, get_callable
from django.views.generic import TemplateView, UpdateView

from profiles.models import Profile
from profiles.utils import class_view_decorator

try:
    ProfileForm = get_callable(settings.PROFILE_FORM_CLASS)
except AttributeError:
    from forms import ProfileForm


@class_view_decorator(login_required)
class ProfileView(TemplateView):
    template_name = 'profiles/profile.html'


@class_view_decorator(login_required)
class ProfileEdit(UpdateView):
    form_class = ProfileForm

    def form_valid(self, form):
        response = super(ProfileEdit, self).form_valid(form)
        messages.info(self.request, 'Your profile has been updated.')
        return response

    def get_context_data(self, **kwargs):
        context = super(ProfileEdit, self).get_context_data(**kwargs)
        context['site'] = Site.objects.get_current()
        return context

    def get_object(self):
        if isinstance(self.request.user, Profile):
            return self.request.user
        return self.request.user.profile

    def get_success_url(self):
        return self.request.GET.get('next', self.success_url or reverse('profile'))




from django.utils.decorators import method_decorator


def class_view_decorator(function_decorator):
    """Convert a function based decorator into a class based decorator usable
    on class based Views.

    Follows the general idea from https://docs.djangoproject.com/en/dev/topics/class-based-views/#decorating-the-class.

    Can't subclass the `View` as it breaks inheritance (super in particular),
    so we monkey-patch instead.
    """

    def simple_decorator(View):
        View.dispatch = method_decorator(function_decorator)(View.dispatch)
        return View

    return simple_decorator



from django import template

from models import Profile

register = template.Library()

@register.inclusion_tag('profiles/_users_count.html')
def users_count():
    return {'users_count': Profile.objects.count()}




from django.db import models
from django.utils.translation import ugettext_lazy as _


class Address(models.Model):
    primary = models.BooleanField(default=False)
    name = models.CharField(max_length=255, verbose_name=_('name'), null=True, blank=True)
    address1 = models.CharField(max_length=255, verbose_name=_('address'), null=True, blank=True)
    address2 = models.CharField(max_length=255, verbose_name=_('address 2'), null=True, blank=True)
    city = models.CharField(max_length=255, verbose_name=_('town/city'), null=True, blank=True)
    region = models.CharField(max_length=255, verbose_name=_('county/state/province'), null=True, blank=True)
    postcode = models.CharField(max_length=15, verbose_name=_('postcode'), null=True, blank=True)
    country = models.ForeignKey('countries.Country', null=True, blank=True)
    telephone = models.CharField(max_length=32, verbose_name=_('telephone'), null=True, blank=True)

    class Meta:
        app_label = 'profiles'
        verbose_name_plural = 'addresses'

    def __unicode__(self):
        return self.name or self.address1



from django.conf import settings
from django.db.models import get_model

from profiles.models import Profile, BasePasswordChangeForm, EmailFormMixin

try:
    User = get_model(*settings.CUSTOM_USER_MODEL.split('.', 2))
except AttributeError:
    from django.contrib.auth.models import User


class UserChangeForm(BasePasswordChangeForm):
    class Meta:
        model = User


class ProfileForm(EmailFormMixin, BasePasswordChangeForm):
    class Meta:
        model = Profile
        exclude = (
            'date_joined',
            'groups',
            'is_active',
            'is_staff',
            'is_superuser',
            'last_login',
            'password',
            'user_permissions',
            'username',
        )
    make_required = ('first_name', 'last_name', 'email',)

    def __init__(self, *args, **kwargs):
        super(ProfileForm, self).__init__(*args, **kwargs)
        for fname in self.make_required:
            try:
                self.fields[fname].required = True
            except KeyError:
                pass



from django.conf.urls.defaults import *

from profiles.views import ProfileEdit, ProfileView


urlpatterns = patterns('profiles.views',
    url(r'^$', ProfileView.as_view(), name='profile'),
    url(r'^edit/$', ProfileEdit.as_view(), name='profile-edit'),
)




from os.path import dirname, join
from setuptools import find_packages, setup

from profiles import get_version

def fread(fname):
    with open(join(dirname(__file__), fname), 'r') as f:
        return f.read()

setup(
    name = "django-extensible-profiles",
    packages = find_packages(),
    include_package_data=True,
    version = get_version(),
    description = "Generic extensible django user profiles.",
    author = "Incuna Ltd",
    author_email = "admin@incuna.com",
    url = "http://incuna.com/",
    long_description=fread("README.markdown"),
)




from django.db import models
from django.utils.translation import ugettext_lazy as _

def register(cls, admin_cls):
    cls.add_to_class('notifications', models.ManyToManyField('profiles.Notification', verbose_name=_('Keep me up-to-date'), 
                                                             null=True, blank=True))

    if admin_cls:
        admin_cls.list_display_filter += ['notifications', ]
        admin_cls.filter_horizontal += ('notifications',)

        if admin_cls.fieldsets:
            admin_cls.fieldsets.append((_('Notifications'), {
                    'fields': ['notifications',],
                    'classes': ('collapse',),
                }))





from django import forms
from django.conf import settings
from django.contrib.auth.models import User, UserManager
from django.core.urlresolvers import get_callable
from django.forms.util import ErrorList
from django.utils.translation import ugettext_lazy as _
from unique_id import generate_id

ModelAdmin = get_callable(getattr(settings, 'PROFILE_MODELADMIN_CLASS', 'django.contrib.admin.ModelAdmin'))
if getattr(settings, 'AUTH_PROFILE_MODULE', False) and settings.AUTH_PROFILE_MODULE == "profiles.Profile":
    def get_profile(self):
        """Returns profile for this user."""
        return self.profile
    User.get_profile = get_profile


class Profile(User):
    objects = UserManager()
    _extensions_imported = False

    @classmethod
    def remove_field(cls, f_name):
        # Removes the field form local fields list
        cls._meta.local_fields = [f for f in cls._meta.local_fields if f.name != f_name]

        # Removes the field setter if exists
        if hasattr(cls, f_name):
            delattr(cls, f_name)

    @classmethod
    def register_extension(cls, register_fn):
        register_fn(cls, ProfileAdmin)

    @classmethod
    def register_extensions(cls, *extensions):
        if not hasattr(cls, '_profile_extensions'):
            cls._profile_extensions = set()

        here = cls.__module__.split('.')[:-1]
        here_path = '.'.join(here + ['extensions'])

        for ext in extensions:
            if ext in cls._profile_extensions:
                continue

            try:
                if isinstance(ext, basestring):
                    try:
                        fn = get_callable(ext + '.register', False)
                    except ImportError:
                        fn = get_callable('%s.%s.register' % ( here_path, ext ), False)
                # Not a string, so take our chances and just try to access "register"
                else:
                    fn = ext.register

                cls.register_extension(fn)
                cls._profile_extensions.add(ext)
            except Exception:
                raise

    def save(self, *args, **kwargs):
        if not self.username:
            self.username = generate_id(first_name=self.first_name,
                                        last_name=self.last_name, email=self.email)
        super(Profile, self).save(*args, **kwargs)

    def get_full_name(self):
        full_name = super(Profile, self).get_full_name()
        if not full_name:
            return self.username
        return full_name

    def get_profile(self):
        return self

# This form is here to avoid a cyclic import between models and forms
class BasePasswordChangeForm(forms.ModelForm):
    """
    Model form that includes password fields and sets password on save.
    """
    password1 = forms.CharField(label=_('Password'), required=False,
                                widget=forms.PasswordInput)
    password2 = forms.CharField(label=_('Password confirmation'),
                                required=False, widget=forms.PasswordInput)

    def clean(self):
       cleaned_data = self.cleaned_data
       password1 = cleaned_data.get('password1', '')
       password2 = cleaned_data.get('password2', '')
       if password1 != password2:
           msg = u'The password fields did not match.'
           self._errors['password2'] = ErrorList([msg])
           if 'password1' in cleaned_data.keys():
               del cleaned_data['password1']
           if 'password2' in cleaned_data.keys():
               del cleaned_data['password2']

       return cleaned_data

    def save(self, commit=True):
       user = super(BasePasswordChangeForm, self).save(commit=False)
       password = self.cleaned_data.get('password1', None)
       if password:
           user.set_password(password)
       if commit:
           user.save()
           self.save_m2m()
       return user


class EmailFormMixin(object):
    """
    Form mixin to ensure that the email is unique.
    """
    def clean_email(self):
        """Prevent account hijacking by disallowing duplicate emails."""
        email = self.cleaned_data.get('email', None)

        if email:
            users = User.objects.filter(email__iexact=email)
            if self.instance:
                users = users.exclude(pk=self.instance.pk)
            if users.count() > 0:
                raise forms.ValidationError(_('That email address is already in use.'))
        return email


class ProfileAdminForm(EmailFormMixin, BasePasswordChangeForm):
    pass


class ProfileAdmin(ModelAdmin):
    fieldsets = [
        (None, {
            'fields': ['email', 'first_name','last_name',  ]
        }),
        (_('Other options'), {
            'classes': ['collapse',],
            'fields': ['is_active', 'last_login', 'date_joined',],
        }),
    ]
    list_display = ['email', 'first_name', 'last_name', ]
    list_display_links = ['email',]
    search_fields = ['email',  'first_name', 'last_name',]
    readonly_fields = ['last_login', 'date_joined', ]
    list_filter = ['is_active', ]
    list_display_filter = []
    search_fields = ['first_name', 'last_name', 'email']
    ordering = ['email',]

    required_fields = ['email', 'first_name', 'last_name']

    def get_form(self, request, obj=None, **kwargs):

        if not issubclass(self.form, ProfileAdminForm):
            # Delay setting the ProfileAdminForm.
            # If form is added to the ProfileAdmin class then it cause 
            # validation error due to the form fields and admin fieldsets 
            # not being consistent.

            self.form = ProfileAdminForm

            # Delay adding the password fields until the form is set.
            fields = self.fieldsets[0][1]['fields']
            try:
                index = fields.index('email') + 1
            except ValueError:
                index = len(fields)
            if not 'password1' in fields:
                fields[index:index] = ['password1', 'password2']

        # Create the form 
        form = super(ProfileAdmin, self).get_form(request, obj=obj, **kwargs)

        # Set required fields
        for fname in self.required_fields:
            form.base_fields[fname].required = True

        return form


# Register extensions listed in the settings
PROFILE_EXTENSIONS = getattr(settings, 'PROFILE_EXTENSIONS', None)
if PROFILE_EXTENSIONS and not Profile._extensions_imported:
    Profile.register_extensions(*PROFILE_EXTENSIONS)
    Profile._extensions_imported = True
